#include "can_open.h"
/*
* @brief Конструктор класса
* @param[in]  QObject предок или QWidget для визуальных компонентов
* @param[out] нет
* @retval нет
*/

CAN_Open::CAN_Open(QWidget *parent): QObject(parent)
{
    m_serial = new QSerialPort(this);
    connect(m_serial, &QSerialPort::readyRead, this, &CAN_Open::readData);
    connect(m_serial, SIGNAL(bytesWritten(qint64)), this, SLOT(writeData(qint64)));
}


/*
* @brief Процедура открытия порта
* @param[in]  QStirng Имя последовательного порта
* @param[out] нет
* @retval Status Error
*/
void CAN_Open::portConnect(QString portName)
{
    m_serial->setPortName(portName);
    m_serial->setBaudRate(QSerialPort::Baud115200);
    m_serial->setDataBits(QSerialPort::Data8);
    m_serial->setStopBits(QSerialPort::OneStop);
    m_serial->setParity(QSerialPort::NoParity);

   /// m_serial->setReadBufferSize(13);
    if (m_serial->open(QIODevice::ReadWrite)) {
            qDebug() << "Connected "+portName;
    } else {
        // Message QString   m_serial->errorString());
    }
}

/*
* @brief Процедура закрытия порта
* @param[in]  нет
* @param[out] нет
* @retval Status Error
*/
void CAN_Open::portDisconnect()
{
   m_serial->disconnect();
   m_serial->close();
   if (!m_serial->isOpen()) {
      qDebug() << "Disconnected " +m_serial->portName();
   }
}

/*
* @brief Процедура записи данных
* @param[in]  QByteArray Буффер пакета CAN open длина 13 байт
* @param[out] нет
* @retval Status Error
*/

void CAN_Open::writeData(QByteArray buff)
{

    m_serial->write(buff);
   // qDebug() << "d write" << m_serial->bytesToWrite();
    while ( m_serial->waitForBytesWritten(3000)) { }

}

/*
* @brief Функция возращает текст ошибка по ее коду
* @param[in]  data32 из буфера чтения
* @param[out] QString текст ошибки
* @retval Status Error
*/

QString CAN_Open::errorString()
{
  uint32_t temp = 0;

  temp =  (m_data[11] << 24);
  temp |= (m_data[10] << 16);
  temp |= (m_data[9] << 8);
  temp |= (m_data[8]);

  qDebug() << "Error code :" <<  temp;

  switch (temp) {
    case 0x05030000: return "Не изменился мерцающий (toggle) бит";
    case 0x05040000: return "Тайм-аут SDO протокола";
    case 0x05040001: return "Неверная или неизвестная команда протокола";
    case 0x06010000: return "Запрашиваемый доступ к объекту не поддерживается";
    case 0x06010001: return "Попытка чтения объекта доступного только для записи";
    case 0x06010002: return "Попытка записи объекта доступного только для чтения";
    case 0x06020000: return "Объект не существует";
    case 0x06040041: return "Объект не может быть отображен в PDO";
    case 0x06040042: return "Полная длина отображаемых объектов превышает максимальный размер PDO (64 бита)";
    case 0x06040043: return "Общая несовместимость параметров";
    case 0x06040047: return "Общая внутренняя несовместимость в устройстве";
    case 0x06060000: return "Отказ в доступе из-за аппаратной ошибки";
    case 0x06070010: return "Неподходящий тип данных или длина сервисного параметра";
    case 0x06090030: return "Значение параметра вне диапазона (только для записи данных)";
    case 0x06090031: return "Значение параметра превышает верхний предел";
    case 0x06090032: return "Значение параметра превышает нижний предел";
    case 0x06090036: return "Максимальное значение меньше минимального";
    case 0x08000000: return "Общая ошибка";
    case 0x08000020: return "Данные не могут быть переданы приложению";
    case 0x08000021: return "Данные не могут быть переданы приложению из-за особенностей локального управления";
    case 0x08000022: return "Данные не могут быть переданы приложению вследствие текущего состояния устройства";
    case 0x08000023: return "Не удалось динамически сгенерировать объектный словарь или нет объектного словаря";
    default:
         return  "unknown Error";
  }
}
/*
* @brief Функция возращает состоянии порта (переопределена от QSerialPort::isOpen)
* @param[in]
* @param[out] Bool
* @retval
*/

bool CAN_Open::isOpen()
{
    return m_serial->isOpen();
}

/*
* @brief Функция ожидания готовности приема данных (переопределена от QSerialPort)
* @param[in]
* @param[out] Bool
* @retval
*/

bool CAN_Open::waitResponce(int ms)
{
   while (m_serial->waitForReadyRead(ms)) { }
   return true;
}

/*
* @brief Слот приема данных (переопределена от QSerialPort)
* @param[in]
* @param[out]
* @retval Буффер  m_data
*/

void CAN_Open::readData()
{
   m_data = m_serial->readAll();
   m_serial->flush();
   emit canReady();
}
/*
* @brief  Слот выводит количество записанных байт
* @param[in]   qint64
* @param[out]  Debug
* @retval  нет
*/

void CAN_Open::writeData(qint64 count)
{
    qDebug() << "serial write " << count;
}
