/* Класс CAN_Open для конфигурирования модулей RealLab
 * --------------------------------------------------
** $QT_BEGIN_LICENSE:LGPL$
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see https://www.qt.io/terms-conditions. For further
** information use the contact form at https://www.qt.io/contact-us.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 3 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL3 included in the
** packaging of this file. Please review the following information to
** ensure the GNU Lesser General Public License version 3 requirements
** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 2.0 or (at your option) the GNU General
** Public license version 3 or any later version approved by the KDE Free
** Qt Foundation. The licenses are as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
** included in the packaging of this file. Please review the following
** information to ensure the GNU General Public License requirements will
** be met: https://www.gnu.org/licenses/gpl-2.0.html and
** https://www.gnu.org/licenses/gpl-3.0.html.
**
** $QT_END_LICENSE$
---------------------------------------------------------------------*/

#ifndef CAN_OPEN_H
#define CAN_OPEN_H


#include <QWidget>
#include <QSerialPort>
#include <QSerialPortInfo>
#include <QDebug>


// Шаблоны запросов CAN LSS
const uint8_t LSS_CONFIG[13] = {0xAE,0x07,0xE5,0x08,0x04,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0xAE};
//Широко вещательная команда. Перевод модуля в режим конфигурирования
const uint8_t LSS_SAVE[13]   = {0xAE,0x07,0xE5,0x08,0x17,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xAE};
//Широко вещательная команда. Сохранения установленных параметров
const uint8_t LSS_SPEED[13]  = {0xAE,0x07,0xE5,0x08,0x13,0x00,0x04,0x00,0x00,0x00,0x00,0x00,0xAE};
//LSS_SPEED[6] изменить согласно таблицы см. РЭ модуля, например = 4 (125 kBit)
const uint8_t LSS_NEW_ID[13] = {0xAE,0x07,0xE5,0x08,0x11,0xFF,0x00,0x00,0x00,0x00,0x00,0x00,0xAE};
//LSS_SPEED[5] изменить на необходимый ID (0-127)
const uint8_t USB_CAN_SPEED[3] = {0xF1,0x04,0xF1};
//USB_CAN_SPEED[2] изменить согласно таблицы см. РЭ модуля, например = 4 (125 kBit)


union CANDataUnit
{
    uint32_t d32;
    float f32;
    uint8_t byte[4];
}; // Данные CAN для удобства хранения и перевода формата


class CAN_Open: public QObject
{
   Q_OBJECT
public:
    explicit CAN_Open(QWidget *parent = nullptr);
    void portConnect(QString portName);
    void portDisconnect();
    void writeData(QByteArray buff);
    QByteArray getData() {return m_data;} // Возвращает буффер
    QString errorString();
    bool isOpen();
    void clearBuffer(){m_serial->flush();} // Быстро отправляем и очищаем буффер
    bool waitResponce(int ms);


public slots:
    void readData();
    void writeData(qint64 count);

signals:
    void canReady();
private:
    QByteArray m_data;
    QSerialPort *m_serial;
};

#endif // CAN_OPEN_H
