#include "can_map.h"

static inline int hexToInt(const QString hex)
{
    bool bStatus = false;
    int res = -1;
    res = hex.toUInt(&bStatus,16);
    if (bStatus) return res;
    return res;
}

static inline QString intToHex(uint data, uint length)
{
    QString temp = "0000000000000000";
    QString str = QString::number(data,16).toUpper();
    int len = length-str.length();
    return temp.left(len)+str;
}




CAN_MapList::CAN_MapList(QObject *parent):
    QAbstractTableModel(parent)

{

}

void CAN_MapList::addFunction(Can_Function *func)
{
    //beginInsertRows(QModelIndex(),index().row(), drow);
    can_list.append(func);
    //endInsertRows();
}

Can_Function *CAN_MapList::getFunction(const QModelIndex &index)
{
    return can_list[index.row()];
}

Can_Function *CAN_MapList::getByRow(int irow)
{
    return can_list[irow];
}

Can_Function *CAN_MapList::getByObjIndex(QString oi)
{
   //return nullptr;
}


void CAN_MapList::loadFromFile(QString name)
{
    QFile file(name);
    file.open(QIODevice::ReadOnly);
    if (file.isOpen()){
        qDebug() << "open";
        QJsonDocument doc = QJsonDocument::fromJson(file.readAll());

        QJsonArray arr = doc.array();
        qDebug() << arr.size();

        can_list.clear();

        //m_rows=0;

        for (int i=0; i<arr.size();i++)
        {
            //ModbusHoldRegs reg;
            //reg.name ="";
            //reg.unit = QModbusDataUnit(QModbusDataUnit::HoldingRegisters,0,1);
            Can_Function *jfunc = new Can_Function();
            jfunc->setFromObj(arr[i].toObject());
            addFunction(jfunc);
           // can_indexRows.insert(jfunc->getObjIndexHex(),i);
        }
        file.close();
    }
}

void CAN_MapList::saveToFile(QString name)
{
    QFile file(name);
    file.open(QIODevice::WriteOnly);
    if (file.isOpen()){

        QJsonDocument doc;
        QJsonArray jsonArr;
        for (int i=0;i<can_list.count();i++)
        {
            jsonArr.append(can_list.at(i)->getObject());
        }
        doc.setArray(jsonArr);
        file.write(doc.toJson());
        file.close();
    }
}

int CAN_MapList::rowCount(const QModelIndex &parent) const
{
    Q_UNUSED(parent);
    return can_list.count();
}

int CAN_MapList::columnCount(const QModelIndex &parent) const
{
    Q_UNUSED(parent);
    return 9;
}

QVariant CAN_MapList::data(const QModelIndex &index, int role) const
{
    Can_Function *func = can_list.at(index.row());
    //static uint indx = func->getObjIndex();

    if (role == Qt::TextAlignmentRole){return Qt::AlignHCenter;}
    else if (role == Qt::DisplayRole || role==Qt::EditRole)  {


        //qDebug() << "Aling";
        switch(index.column())
        {
        case 0: return intToHex(func->getObjIndex(),4);
        case 1: return intToHex(func->getSubIndex(),2);
        case 2: return func->getName(); //.toUtf8();
        case 3: return RW_ModeStr[func->getRWMode()];
        case 4: return Data_TypeStr[func->getDataType()];
        case 5: return func->getValue();
        case 6: return func->getObjMap();
        case 7: return func->getObj10();
        case 8: return func->getObj11();
        default:
            return QVariant();
        }

    }

    else if(role == Qt::BackgroundRole)
    {

        if (func->getIndex() == "Index"){
            return QBrush(QColor(217,219,216),Qt::SolidPattern);
         }
        //return QVariant(QColor(Qt::blue));
    }

    //   if (role == Qt::BackgroundColorRole)
    //    {
    //       return QVariant(QColor(Qt::blue));
    //    }

    return QVariant();
}


QVariant CAN_MapList::headerData(int section, Qt::Orientation orientation, int role) const
{
    if(role!= Qt::DisplayRole)
        return QVariant();
    if(orientation==Qt::Horizontal)
    {
        switch(section)
        {
        case 0: return "Индекс\n объекта";
        case 1: return "Субиндекс\n объекта";
        case 2: return "Имя объекта";
        case 3: return "R/W";
        case 4: return "Тип данных";
        case 5: return "Значение \n по умолчанию";
        case 6: return "Сопоставляемый объект";
        case 7: return "Запись объектом \n 1010h";
        case 8: return "Загрузка объектом \n 1011h";




        }

    }
}


//=======================================
Can_Function::Can_Function()
{

}

QJsonObject Can_Function::getObject()
{
    return QJsonObject::fromVariantMap(map);
}

void Can_Function::setFromObj(QJsonObject obj)
{
    map = obj.toVariantMap();
}

QString Can_Function::getObjIndexHex()
{
    return intToHex(map["ObjIndex"].toUInt(),4)+"h";
}

//void Can_Function::setFunction(uint16_t iobj, uint8_t sobj, QString name, int rwmode, int dtype, QVariant value)
//{

//    map.insert("ObjIndex",iobj);
//    map.insert("SubIndex",sobj);
//    map.insert("ObjName",name);
//    map.insert("RWMode",(int)rwmode);
//    map.insert("DataType",(int)dtype);
//    map.insert("Value",value) ;

//}

void Can_Function::setObjIndex(QString ss)
{
    map.insert("ObjIndex",hexToInt(ss.left(4)));
}

QString Can_Function::getSubIndexHex()
{
   return "0x"+intToHex(map["SubIndex"].toUInt(),2);
}

void Can_Function::setSubIndex(QString ss)
{
    map.insert("SubIndex",hexToInt(ss.left(2)));
}

void Can_Function::setName(QString ss)
{
    map.insert("ObjName",ss);
}

QString Can_Function::getRWModeStr()
{

   int mod = getRWMode();
   switch (mod) {
     case 1: return "RW";
     case 2: return "WO";
     case 3: return "RO";
     default:
       return "";
   }
}

void Can_Function::setRWMode(QString ss)
{
    int t;
    if(ss.contains("RW")) t=1;
    else if(ss.contains("WO")) t=2;
    else if(ss.contains("RO")) t=3;
    else t=0;
    map.insert("RWMode",t);
}

QString Can_Function::getDataTypeStr()
{
    int t = getDataType();
    switch (t) {
      case 1: return "uint8";
      case 2: return "uint16";
      case 3: return "uint32";
      case 4: return "str";
      case 5: return "float";
      default:
        return "--";
    }
}

void Can_Function::setDatType(QString ss)
{
    int dat_t;
    if(ss.contains("uint8")) dat_t=1;
    else if(ss.contains("uint16")) dat_t=2;
    else if(ss.contains("uint32")) dat_t=3;
    else if(ss.contains("float")) dat_t=5;
    else dat_t=0;

    map.insert("DataType",dat_t);
}

void Can_Function::setValue(QString ss)
{
    map.insert("Value",ss);
}

void Can_Function::setObjMap(QString ss)
{
    map.insert("ObjMap",ss);
}

void Can_Function::setObj10(QString ss)
{
    map.insert("1010",ss);
}

void Can_Function::setObj11(QString ss)
{
    map.insert("1011",ss);
}

void Can_Function::setIndex(QString ss)
{
   map.insert("Index",ss);
}
