#ifndef CAN_MAP_H
#define CAN_MAP_H

#include <QObject>
#include <QAbstractTableModel>
#include <QMap>
#include <QList>

#include <QFile>
#include <QJsonArray>
#include <QJsonDocument>
#include <QJsonObject>
#include <QDebug>
#include <QColor>
#include <QBrush>


#include "can_object.h"

enum RW_Mode{NoMode,RW,WO,RO};
enum Data_Type{uncom,data8,data16,data32,dataStr,dataFloat};


const QString RW_ModeStr[4] = {"--","RW","WO","RO"};
const QString Data_TypeStr[6] = {"uncom","uint8","uint16","uint32","string","float"};

//static inline QString intToHex(uint data, uint length);

class Can_Function
{

public:
   Can_Function();

   QJsonObject getObject();
   void setFromObj(QJsonObject obj);

  // void setFunction(uint16_t iobj,uint8_t sobj,QString name,int rwmode, int dtype, QVariant value);

   QString  getObjIndexHex();
   QString  getObjIndexStr(){return map["ObjIndex"].toString();}
   uint16_t getObjIndex(){return map["ObjIndex"].toUInt();}
   void setObjIndex(QString ss);

   QString  getSubIndexHex();
   QString  getSubIndexStr(){return map["SubIndex"].toString();}
   uint8_t getSubIndex(){return map["SubIndex"].toUInt();}
   void setSubIndex(QString ss);

   QString getName(){return map["ObjName"].toString();}
   void setName(QString ss);

   QString getRWModeStr();
   int getRWMode(){return map["RWMode"].toInt();}
   void setRWMode(QString ss);

   QString getDataTypeStr();
   int getDataType(){return map["DataType"].toInt();}
   void setDatType(QString ss);

   QString getValue(){return map["Value"].toString();}
   void setValue(QString ss);

   QString getObjMap(){return map["ObjMap"].toString();}
   void setObjMap(QString ss);

   QString getObj10(){return map["1010"].toString();}
   void setObj10(QString ss);

   QString getObj11(){return map["1011"].toString();}
   void setObj11(QString ss);


   QString getIndex(){return map["Index"].toString();}
   void setIndex(QString ss);

   QVariantMap getMap() {return map;}
private:
   QVariantMap map;

};


class CAN_MapList:public QAbstractTableModel
{
   Q_OBJECT
public:
   explicit CAN_MapList(QObject *parent);
   void addFunction(Can_Function *func);

   Can_Function* getFunction(const QModelIndex &index);
   Can_Function* getByRow(int irow);
   Can_Function* getByObjIndex(QString oi);

   void loadFromFile(QString name);
   void saveToFile(QString name);
   int getListCount() { return can_list.count();}

protected:
    QList <Can_Function*> can_list;


    // QAbstractItemModel interface
public:

    int rowCount(const QModelIndex &parent) const;
    int columnCount(const QModelIndex &parent) const;
    QVariant data(const QModelIndex &index, int role) const;
    QVariant headerData(int section, Qt::Orientation orientation, int role) const;
private:
    QList <CAN_Object*> obj_list;
};



#endif // CAN_MAP_H
