#include "mainwindow.h"
#include "ui_mainwindow.h"

#include "can_open.h"



static inline int hexToInt(const QString hex)
{
   bool bStatus = false;
   int res = -1;
   res = hex.toUInt(&bStatus,16);
   if (bStatus) return res;
   return res;
}


static inline QString intToHex(uint data, uint length)
{
    QString temp = "0000000000000000";
    QString str = QString::number(data,16).toUpper();
    int len = length-str.length();
    return temp.left(len)+str;
}


static inline QString byteToStr(CANDataUnit dat)
{
   QString result="";
   for (int i=0;i<4;i++){
      if (dat.byte[i] < 0x20) continue;
      if (dat.byte[i] > 0x7F) continue;
      result.append(QChar(dat.byte[i]));
   }
   return result;
}

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent)
    , ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    QStringList list;
    auto infos = QSerialPortInfo::availablePorts();
    for (const QSerialPortInfo &info : infos) {
        list << info.portName();
    }
    ui->comboBox->addItems(list);


   // can_model = new CAN_MapList(this);
    settings = new QSettings("RealLab","Can_Config",this);
    loadConfig();

    onOpenFileXML(pathFile);
    ui->lbModul->setText("Модуль "+curModulName);
    ui->edID->setText(QString::number(curID));

    can = new CAN_Open(this);
    connect(can,&CAN_Open::canReady,this,&MainWindow::Responce);

    findTimer = new QTimer(this);
    connect(findTimer,&QTimer::timeout,this,&MainWindow::onFindTimer);

    ui->edObjName->setStyleSheet("color:blue" );
    ui->edValue->setStyleSheet("color:blue");

    ui->cbSeed->setCurrentIndex(4);
}


MainWindow::~MainWindow()
{
    saveConfig();
    delete ui;
}


void MainWindow::Responce()
{
    m_buffer = can->getData();

    if (curReadMode == READ_ID){
       id_items.insert(idCount,m_buffer);
       idCount++;
    }

    curData.d32 = 0;
    curData.byte[3] = m_buffer[11];
    curData.byte[2] = m_buffer[10];
    curData.byte[1] = m_buffer[9];
    curData.byte[0] = m_buffer[8];

    curError = m_buffer[4];

    if (curError != 0x80){
       ui->edValue->setText(QString::number(curData.d32));
       ui->listWidget->addItem("RS: "+getBufferStr(m_buffer));
       if (curReadMode == READ_NAME)
       {
          name_list.append(byteToStr(curData));
          findTimer->start();
       }

    } else {
       ui->listWidget->addItem("RS: "+getBufferStr(m_buffer));
       ui->listWidget->addItem("ER: "+can->errorString());
    }
    can->clearBuffer();
}

void MainWindow::onFindTimer()
{

   if (idCount >= id_list.count()) {
      curReadMode = READ_ALL;
      findTimer->stop();
      onListName();
      return;
   }
   // qDebug() << id_list.at(idCount);

    switch (curIndex) {
    case 0x1008:{
        findTimer->stop();
        onReadData(id_list.at(idCount),curIndex,0x00);
        curIndex=0x1009;
        break;
       }
    case 0x1009:{
        findTimer->stop();
        onReadData(id_list.at(idCount),curIndex,0x00);
        curIndex=0x1008;
        idCount++;
        break;
       }
    }
}

void MainWindow::onOpenFileXML(QString fname)
{
   QFile file(fname);
   QDomDocument doc;
   if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
   {
        qDebug() << "Failed open file...";
        return;
   }
    else
   {
     if (!doc.setContent(&file))
      {
            qDebug() << "Failed load XML file...";
            return;
      }
        file.close();
        statusBar()->showMessage(fname);
  }
  onLoadXML(doc.firstChildElement());
  qDebug() << "Load file finished ok";

}

void MainWindow::onLoadXML(QDomElement root)
{
    ui->treeWidget->clear();
    QStringList hl;
    hl<<"Объекты ммодулей";
    ui->treeWidget->setHeaderLabels(hl);

    QDomNodeList  modulNodes = root.elementsByTagName("Modul");
    qDebug() << "Moduls " << modulNodes.count();
    for (int i=0;i<modulNodes.count();i++)
    {
        QDomNode itemnode = modulNodes.at(i);
        if(itemnode.isElement())
        {
            QDomElement itemelem = itemnode.toElement();
            QTreeWidgetItem *ti = new QTreeWidgetItem(0);
            ti->setText(0,itemelem.attribute("Name"));
            ui->treeWidget->addTopLevelItem(ti);

            QDomNodeList  IndexNodes = itemelem.elementsByTagName("ObjFunc");
            for (int n=0;n<IndexNodes.count();n++)
            {

                QDomNode funcnode = IndexNodes.at(n);
                if(funcnode.isElement())
                {
                    QDomElement felem = funcnode.toElement();

                    QString str=felem.attribute("ObjIndex");
                    str += ("_" + felem.attribute("ObjName"));
                    str += ("_" + felem.attribute("Mode"));
                    str += ("_" + felem.attribute("DataType"));
                    str += ("_" + felem.attribute("ObjMap"));

                    QTreeWidgetItem *root = ui->treeWidget->topLevelItem(i);
                    QTreeWidgetItem *ti = new QTreeWidgetItem(0);
                    ti->setText(0,str);
                    root->addChild(ti);



                    QDomNodeList  SubIndexs = felem.elementsByTagName("SubFunc");
                    for (int sn=0;sn<SubIndexs.count();sn++)
                    {

                        QDomNode subfunc = SubIndexs.at(sn);
                        if(subfunc.isElement())
                        {
                            QDomElement selem = subfunc.toElement();
                            QString str=selem.attribute("SubIndex");
                            str += (";" + selem.attribute("SubName"));
                            str += (";" + selem.attribute("Mode"));
                            str += (";" + selem.attribute("DataType"));
                            str += (";" + selem.attribute("ObjMap"));

                            QTreeWidgetItem *sub = new QTreeWidgetItem(0);
                            sub->setText(0,str);
                            ti->addChild(sub);
                        }
                    }
                }
            }
        }
    }
}

void MainWindow::onFindModulView()
{
    curReadMode = READ_ALL;
    QString  str= "Find %1 moduls";
    int N=id_items.count();
    statusBar()->showMessage(str.arg(N));
    ui->mList->clear();
    id_list.clear();
    for (int i=0;i<N;i++)
    {
        QByteArray buf = id_items[i];
        uint16_t bufid = 0;
        bufid = (buf[1] << 8) | buf[2];
        id_list.append(bufid);
        ui->mList->addItem(QString::number(bufid & 0x7F)+"-ID");
    }
}

void MainWindow::onListName()
{
   ui->mList->clear();
   for (int i=0;i<id_list.count();i++)
   {
      ui->mList->addItem(QString::number(id_list.at(i))+"-"+name_list.at(2*i)+name_list.at(2*i+1));
   }

}



void MainWindow::onReadData(uint id, uint index, uint subindex)
{
    m_buffer.clear();
    m_buffer.append(0xAE);
    m_buffer.append(0x06);
    m_buffer.append(id);
    m_buffer.append(0x08);
    m_buffer.append(0x40);
    m_buffer.append(index & 0xFF);
    m_buffer.append((index >> 8) & 0xFF);
    m_buffer.append(subindex);
    m_buffer.append((QChar)0x00);
    m_buffer.append((QChar)0x00);
    m_buffer.append((QChar)0x00);
    m_buffer.append((QChar)0x00);
    m_buffer.append(0xAE);
    can->writeData(m_buffer);
    ui->listWidget->addItem("RD: "+intToHex(index,4)+" "+getBufferStr(m_buffer));
}

void MainWindow::onWriteData(uint id, uint index, uint subindex, uint32_t data)
{
    m_buffer.clear();
    m_buffer.append(0xAE);
    m_buffer.append(0x06);
    m_buffer.append(id);
    m_buffer.append(0x08);
    m_buffer.append(0x22);
    m_buffer.append(index & 0xFF);
    m_buffer.append((index >> 8) & 0xFF);
    m_buffer.append(subindex);
    m_buffer.append(data & 0xFF);
    m_buffer.append((data >> 8) & 0xFF);
    m_buffer.append((data >> 16) & 0xFF);
    m_buffer.append((data >> 24)  & 0xFF);
    m_buffer.append(0xAE);
    can->writeData(m_buffer);
    ui->listWidget->addItem("WR: "+intToHex(index,4)+" "+getBufferStr(m_buffer));
}

QByteArray MainWindow::copyPacket(const uint8_t paket[], int len)
{
    QByteArray res;
    for (int i=0;i < len;i++) {
       // qDebug() << intToHex(paket[i],2);
        res.append(paket[i]);
    }
    return res;
}


void MainWindow::saveConfig()
{
   settings->setValue("PathLastFile",pathFile);
   settings->setValue("CurModulName",curModulName);
   settings->setValue("CurID",curID);
}

void MainWindow::loadConfig()
{
   pathFile = settings->value("PathLastFile","").toString();
   curModulName = settings->value("CurModulName","---").toString();
   curID = settings->value("CurID",0).toUInt();
   qDebug() << curModulName << " " << curID;
}


void MainWindow::on_btOpen_clicked()
{


  if (!can->isOpen())
  {
      QString pname = ui->comboBox->currentText();
      can->portConnect(pname);
      statusBar()->showMessage("Открытие порта: " + pname, 5000 );
      ui->btOpen->setText("Закрыть порт");
  } else {
      can->portDisconnect();
      ui->btOpen->setText("Открыть порт");
      statusBar()->showMessage("Закрытие порт", 5000 );
  }

}

void MainWindow::on_btWrite_clicked()
{
    if (!can->isOpen())
      {
        QMessageBox::warning(this,"USB-CAN порт","Порт закрыт!!!");
        return;
      }

    onWriteData(curID,curIndex,curSubIndex,curData.d32);
    ui->listWidget->addItem("WR: "+ getBufferStr(m_buffer));
}

QString MainWindow::getBufferStr(QByteArray arr)
{
  return arr.toHex().toUpper();

}



void MainWindow::on_actionOpenFile_triggered()
{
   pathFile = QFileDialog::getOpenFileName(this,"Открыть файл объектов","","*xml");
   onOpenFileXML(pathFile);
}



void MainWindow::on_treeWidget_itemClicked(QTreeWidgetItem *item, int column)
{
    QString idstr=item->text(column); //int id = item->text()
    statusBar()->showMessage(idstr);

    if (idstr.contains("NLS")) {
      ui->lbModul->setText("Модуль: "+idstr);
      return;
    }

    QStringList sl;
    if (idstr.contains("_")) sl = idstr.split("_");
      else if  (idstr.contains(";")) sl = idstr.split(";");

    if (sl.count() < 4) return;



   if (idstr.contains("_"))
   {
      ui->edObjIndex->setText(sl[0]);
      ui->edObjName->setText(sl[1]);
      ui->edSubIndex->setText("00h");
      ui->edMode->setText(sl[2]);
      ui->edDataType->setText(sl[3]);
      ui->edObjMap->setText(sl[4]);

      curIndex = hexToInt(sl[0].left(4));
      curSubIndex = 0;

   }

   if (idstr.contains(";"))
   {
      ui->edObjIndex->setText(intToHex(curIndex,4)+"h");
      ui->edObjName->setText(sl[1]);
      ui->edSubIndex->setText(sl[0]);
      ui->edMode->setText(sl[2]);
      ui->edDataType->setText(sl[3]);
      ui->edObjMap->setText(sl[4]);
      curSubIndex = hexToInt(sl[0].left(2));

   }

   ui->btRead->setEnabled(false);
   ui->btWrite->setEnabled(false);

   if ((sl[2] == "RW") || (sl[2] == "WO")) ui->btWrite->setEnabled(true);
   if ((sl[2] == "RW") || (sl[2] == "RO")) ui->btRead->setEnabled(true);

}

void MainWindow::on_cbModuls_currentTextChanged(const QString &arg1)
{
    curModulName = arg1;
}

void MainWindow::on_treeWidget_itemExpanded(QTreeWidgetItem *item)
{
   on_treeWidget_itemClicked(item,0);
}

void MainWindow::on_edValue_textChanged(const QString &arg1)
{
    QString dstr;
    dstr = ui->edValue->text();
    if (arg1.contains(".")) {
      curData.f32 = dstr.toFloat();
    } else {
      curData.d32 = dstr.toInt();
    }
    ui->edHex->setText("0x"+intToHex((uint)curData.d32,8));
    ui->edStr->setText(byteToStr(curData));
}

void MainWindow::on_btRead_clicked()
{
    if (!can->isOpen())
      {
        QMessageBox::warning(this,"USB-CAN порт","Порт закрыт!!!");
        return;
      }
    m_buffer.clear();
    curReadMode = READ_ALL;
    onReadData(curID,curIndex,curSubIndex);
    ui->listWidget->addItem("RD: "+ getBufferStr(m_buffer));
}

void MainWindow::on_btClearList_clicked()
{
    ui->listWidget->clear();
}

void MainWindow::on_actionSetStop_triggered()
{
    if (!can->isOpen())
      {
        QMessageBox::warning(this,"USB-CAN порт","Порт закрыт!!!");
        return;
      }

    m_buffer.clear();
    m_buffer.append(0xAE); // 0
    m_buffer.append((QChar)0x00); // 1
    m_buffer.append((QChar)0x00); // 2
    m_buffer.append(0x02); // 3
    m_buffer.append(0x02); // 4
    m_buffer.append(curID & 0x7F); // 5
    m_buffer.append((QChar)0x00); // 6
    m_buffer.append((QChar)0x00); // 7
    m_buffer.append((QChar)0x00); // 8
    m_buffer.append((QChar)0x00); // 9
    m_buffer.append((QChar)0x00); // 10
    m_buffer.append((QChar)0x00); // 11
    m_buffer.append(0xAE); // 12

    can->writeData(m_buffer);
    ui->listWidget->addItem("WR: "+ getBufferStr(m_buffer));

}

void MainWindow::on_actionEXit_triggered()
{
    close();
}

void MainWindow::on_actionSetConfigMode_triggered()
{

    if (!can->isOpen())
      {
        QMessageBox::warning(this,"USB-CAN порт","Порт закрыт!!!");
        return;
      }

    m_buffer.clear();
    m_buffer.append(0xAE); // 0
    m_buffer.append((QChar)0x00); // 1
    m_buffer.append((QChar)0x00); // 2
    m_buffer.append(0x02); // 3
    m_buffer.append(0x80); // 4
    m_buffer.append(curID & 0x7F); // 5
    m_buffer.append((QChar)0x00); // 6
    m_buffer.append((QChar)0x00); // 7
    m_buffer.append((QChar)0x00); // 8
    m_buffer.append((QChar)0x00); // 9
    m_buffer.append((QChar)0x00); // 10
    m_buffer.append((QChar)0x00); // 11
    m_buffer.append(0xAE); // 12

    can->writeData(m_buffer);
     ui->listWidget->addItem("WR: "+ getBufferStr(m_buffer));
}

void MainWindow::on_actionSetWorkMode_triggered()
{
    if (!can->isOpen())
      {
        QMessageBox::warning(this,"USB-CAN порт","Порт закрыт!!!");
        return;
      }

    m_buffer.clear();
    m_buffer.append(0xAE); // 0
    m_buffer.append((QChar)0x00); // 1
    m_buffer.append((QChar)0x00); // 2
    m_buffer.append(0x02); // 3
    m_buffer.append(0x01); // 4
    m_buffer.append(curID & 0x7F); // 5
    m_buffer.append((QChar)0x00); // 6
    m_buffer.append((QChar)0x00); // 7
    m_buffer.append((QChar)0x00); // 8
    m_buffer.append((QChar)0x00); // 9
    m_buffer.append((QChar)0x00); // 10
    m_buffer.append((QChar)0x00); // 11
    m_buffer.append(0xAE); // 12

    can->writeData(m_buffer);
     ui->listWidget->addItem("WR: "+ getBufferStr(m_buffer));

}

void MainWindow::on_actionSetID_triggered()
{

    QMessageBox::StandardButton reply;
    reply = QMessageBox::question(this, "Изменение ID модуля", "Внимание! Групповая команда.\r\nID будут установленно на всех модулях.\r\nВы Уверены?",
                                    QMessageBox::No|QMessageBox::Yes);
    if (reply != QMessageBox::Yes) {
        qDebug() << "No change ID";
        return;
    }

    if (!can->isOpen())
      {
        QMessageBox::warning(this,"USB-CAN порт","Порт закрыт!!!");
        return;
      }

    curReadMode=READ_ALL;

    //QThread::msleep(500);

    m_buffer.clear();
    m_buffer = copyPacket(LSS_CONFIG,13);
    qDebug() << m_buffer;
    can->writeData(m_buffer);

    ui->listWidget->addItem("WR: "+ getBufferStr(m_buffer));
    //can->clearBuffer();
    //QThread::msleep(500);

    //can->waitResponce(200);

    m_buffer.clear();
    m_buffer = copyPacket(LSS_NEW_ID,13);
    m_buffer[5] = ui->edNewID->text().toUInt() & 0x7F; // 6
    can->writeData(m_buffer);
    ui->listWidget->addItem("WR: "+ getBufferStr(m_buffer));
    //can->clearBuffer();

    can->waitResponce(1000);

    m_buffer.clear();
    m_buffer = copyPacket(LSS_SAVE,13);
    can->writeData(m_buffer);
    ui->listWidget->addItem("WR: "+ getBufferStr(m_buffer));
    can->clearBuffer();





}

void MainWindow::on_actionSetSpeed_triggered()
{
    if (!can->isOpen())
      {
        QMessageBox::warning(this,"USB-CAN порт","Порт закрыт!!!");
        return;
      }

    QMessageBox::StandardButton reply;
    reply = QMessageBox::question(this, "Изменение скорости CAN", "Внимание! Групповая команда.\r\nДействие будует применено\r ко всем модулям. Вы Уверены?",
                                    QMessageBox::Yes|QMessageBox::No);
    if (reply != QMessageBox::Yes) {
        qDebug() << "No change speed";
        return;
    }

    curReadMode=READ_ALL;

    //QThread::msleep(500);

    m_buffer.clear();
    m_buffer = copyPacket(LSS_CONFIG,13);
    qDebug() << m_buffer;

    ui->listWidget->addItem("WR: "+ getBufferStr(m_buffer));
    can->writeData(m_buffer);

    can->waitResponce(500);

    m_buffer.clear();
    m_buffer = copyPacket(LSS_SPEED,13);
    m_buffer[6] = ui->cbSeed->currentIndex() & 0x0F; // 6
    ui->listWidget->addItem("WR: "+ getBufferStr(m_buffer));
    can->writeData(m_buffer);
    can->waitResponce(1000);

    m_buffer.clear();
    m_buffer = copyPacket(LSS_SAVE,13);

    ui->listWidget->addItem("WR: "+ getBufferStr(m_buffer));
    can->writeData(m_buffer);
    can->waitResponce(3500);

    m_buffer.clear();
    m_buffer = copyPacket(USB_CAN_SPEED,3);
    m_buffer[1] = ui->cbSeed->currentIndex() & 0x0F; // 6
    ui->listWidget->addItem("WR: "+ getBufferStr(m_buffer));
    can->writeData(m_buffer);

}



void MainWindow::on_edID_textChanged(const QString &arg1)
{
     curID = arg1.toUInt();
}

void MainWindow::on_btStart_clicked()
{
    if (!can->isOpen())
      {
        QMessageBox::warning(this,"USB-CAN порт","Порт закрыт!!!");
        return;
      }

    ui->listWidget->clear();
    m_buffer.clear();

    m_buffer.append(0xAE); // 0
    m_buffer.append((QChar)0x00); // 1
    m_buffer.append((QChar)0x00); // 2
    m_buffer.append(0x02); // 3
    m_buffer.append(0x81); // 4
    m_buffer.append((QChar)0x00); // 5
    m_buffer.append((QChar)0x00); // 6
    m_buffer.append((QChar)0x00); // 7
    m_buffer.append((QChar)0x00); // 8
    m_buffer.append((QChar)0x00); // 9
    m_buffer.append((QChar)0x00); // 10
    m_buffer.append((QChar)0x00); // 11
    m_buffer.append(0xAE); // 12

    curReadMode = READ_ID;
    idCount=0;

    id_list.clear();
    id_items.clear();
    can->writeData(m_buffer);
    ui->listWidget->addItem("Get ID: "+ getBufferStr(m_buffer));
    QTimer::singleShot(1000,this,onFindModulView);
}

void MainWindow::on_btGetName_clicked()
{
    if (id_list.count() < 1) {
        QMessageBox::warning(this,"Поиск имен","Список ID пуст!!!");
        return;
    }

    ui->listWidget->clear();
    idCount=0;
    curReadMode = READ_NAME;
    curIndex=0x1008;
    findTimer->setInterval(300);
    findTimer->start();

}

void MainWindow::on_mList_itemClicked(QListWidgetItem *item)
{
    QStringList sl = item->text().split("-");
    curID = sl[0].toUInt();
    ui->edID->setText(sl[0]);
}

void MainWindow::on_actionReStart_triggered()
{

    if (!can->isOpen())
      {
        QMessageBox::warning(this,"USB-CAN порт","Порт закрыт!!!");
        return;
      }

    m_buffer.clear();
    m_buffer.append(0xAE); // 0
    m_buffer.append((QChar)0x00); // 1
    m_buffer.append((QChar)0x00); // 2
    m_buffer.append(0x02); // 3
    m_buffer.append(0x81); // 4
    m_buffer.append(curID &0x7F); // 5
    m_buffer.append((QChar)0x00); // 6
    m_buffer.append((QChar)0x00); // 7
    m_buffer.append((QChar)0x00); // 8
    m_buffer.append((QChar)0x00); // 9
    m_buffer.append((QChar)0x00); // 10
    m_buffer.append((QChar)0x00); // 11
    m_buffer.append(0xAE); // 12

    can->writeData(m_buffer);
    ui->listWidget->addItem("WR: "+ getBufferStr(m_buffer));

}

void MainWindow::on_btWrSpeed_clicked()
{
    on_actionSetSpeed_triggered();
}

void MainWindow::on_btWrID_clicked()
{
    on_actionSetID_triggered();
}

void MainWindow::on_listWidget_itemEntered(QListWidgetItem *item)
{
    ui->listWidget->item(ui->listWidget->count()-1)->setSelected(true);
}

void MainWindow::on_pushButton_clicked()
{
    QMessageBox::StandardButton reply;
    reply = QMessageBox::question(this, "Изменение скорости USB-CAN", "Вы Уверены?",
                                    QMessageBox::Yes|QMessageBox::No);
    if (reply != QMessageBox::Yes) {
        qDebug() << "No change speed";
        return;
    }

    if (!can->isOpen())
      {
        QMessageBox::warning(this,"USB-CAN порт","Порт закрыт!!!");
        return;
      }

    m_buffer.clear();
    m_buffer.append(0xF1); // 0
    m_buffer.append(ui->cbSeed->currentIndex() & 0x0F); // 3
    m_buffer.append(0xF1); // 4
    can->writeData(m_buffer);
    can->clearBuffer();
    ui->listWidget->addItem("Speed USB_CAN: "+ getBufferStr(m_buffer));
}

void MainWindow::on_btSaveUp_clicked()
{
    curData.d32 = 0x65766173;
    onWriteData(curID,0x1010,0x01,curData.d32);
    on_btWrite_clicked();
}

void MainWindow::on_btLoadPar_clicked()
{
    curData.d32 = 0x64616F6C;
    onWriteData(curID,0x1011,0x01,curData.d32);
    on_btWrite_clicked();
}

void MainWindow::on_btStopMode_clicked()
{
    on_actionSetStop_triggered();
}

void MainWindow::on_btResetMode_clicked()
{
   on_actionReStart_triggered();
}

void MainWindow::on_btConfMode_clicked()
{
  on_actionSetConfigMode_triggered();
}

void MainWindow::on_btWorkMode_clicked()
{
   on_actionSetWorkMode_triggered();
}

