#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QFileDialog>
#include <QSettings>
#include <QMessageBox>
#include <QTreeWidgetItem>
#include <QListWidgetItem>
#include <QTimer>
#include <QtXml>

#include "can_open.h"

#define READ_ALL  0
#define READ_ID   1
#define READ_NAME 2



const QString canModulStr[8] =
{
  "NLS-16DO-CAN",
  "NLS-16DI-CAN",
  "NLS-8AI-CAN",
  "NLS-8AI-I-CAN",
  "NLS-8R-CAN",
  "NLS-4AO-CAN",
  "NLS-4RTD-CAN",
  "NLS-8TI-CAN",
};


QT_BEGIN_NAMESPACE
namespace Ui { class MainWindow; }
QT_END_NAMESPACE

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    MainWindow(QWidget *parent = nullptr);
    ~MainWindow();

public slots:
   void Responce();

private slots:
    void onFindTimer();
    void onOpenFileXML(QString fname);
    void onLoadXML(QDomElement root);
    void onFindModulView();
    void onListName();

    //-----------------------------
    void onReadData(uint id, uint index,uint subindex);
    void onWriteData(uint id, uint index,uint subindex, uint32_t data);
    QByteArray copyPacket(const uint8_t paket[] , int len);

    //-----------------------------
    void saveConfig();
    void loadConfig();

    //-----------------------------
    void on_btOpen_clicked();
    void on_btWrite_clicked();
    void on_btRead_clicked();
    void on_btStart_clicked();
    //-----------------------------------
    QString getBufferStr(QByteArray arr);
    void on_treeWidget_itemClicked(QTreeWidgetItem *item, int column);
    void on_cbModuls_currentTextChanged(const QString &arg1);

    void on_treeWidget_itemExpanded(QTreeWidgetItem *item);

    void on_edValue_textChanged(const QString &arg1);

    void on_btClearList_clicked();

    void on_actionOpenFile_triggered();
    void on_actionSetStop_triggered();
    void on_actionEXit_triggered();
    void on_actionSetConfigMode_triggered();
    void on_actionSetWorkMode_triggered();
    void on_actionSetID_triggered();
    void on_actionSetSpeed_triggered();
    void on_edID_textChanged(const QString &arg1);

    void on_btGetName_clicked();

    void on_mList_itemClicked(QListWidgetItem *item);

    void on_actionReStart_triggered();

    void on_btWrSpeed_clicked();

    void on_btWrID_clicked();

    void on_listWidget_itemEntered(QListWidgetItem *item);

    void on_pushButton_clicked();

    void on_btSaveUp_clicked();

    void on_btLoadPar_clicked();

    void on_btStopMode_clicked();

    void on_btResetMode_clicked();

    void on_btConfMode_clicked();

    void on_btWorkMode_clicked();



private:

    uint16_t curIndex;
    uint16_t curSubIndex;
    CANDataUnit curData;
    uint16_t curID;
    uint8_t curError;
    int curReadMode = READ_ALL;

    QString pathFile;
    QString curModulName;
   // TreeModel *tree_model;
    Ui::MainWindow *ui;
    QSettings *settings;
    uint16_t curFunc;
    CAN_Open *can;
    QTimer *findTimer;
    uint8_t m_data=0;
    QByteArray m_buffer;
    QMap<uint,QByteArray>  id_items;
    QList<uint8_t> id_list;
    QStringList  name_list;
    uint8_t idCount=0;
};
#endif // MAINWINDOW_H
